package com.nativelibs4java.opencl;
import org.bridj.*;
import org.bridj.util.*;
import java.lang.reflect.Type;
/**
 * Size in bytes of a __local argument.
 */
public class LocalSize {
	long size;
	public LocalSize(long size) {
		this.size = size;
	}
	
	
	/**
	 * Returns the size in bytes of an array of int values of the specified length.<br>
	 * @return <code>arrayLength * sizeof(int) = arrayLength * 4<br>
	 */
	public static LocalSize ofIntArray(long arrayLength) {
		return new LocalSize(arrayLength * 4);
	}
	
	
	/**
	 * Returns the size in bytes of an array of long values of the specified length.<br>
	 * @return <code>arrayLength * sizeof(long) = arrayLength * 8<br>
	 */
	public static LocalSize ofLongArray(long arrayLength) {
		return new LocalSize(arrayLength * 8);
	}
	
	
	/**
	 * Returns the size in bytes of an array of short values of the specified length.<br>
	 * @return <code>arrayLength * sizeof(short) = arrayLength * 2<br>
	 */
	public static LocalSize ofShortArray(long arrayLength) {
		return new LocalSize(arrayLength * 2);
	}
	
	
	/**
	 * Returns the size in bytes of an array of byte values of the specified length.<br>
	 * @return <code>arrayLength * sizeof(byte) = arrayLength * 1<br>
	 */
	public static LocalSize ofByteArray(long arrayLength) {
		return new LocalSize(arrayLength * 1);
	}
	
	
	/**
	 * Returns the size in bytes of an array of char values of the specified length.<br>
	 * @return <code>arrayLength * sizeof(char) = arrayLength * 2<br>
	 */
	public static LocalSize ofCharArray(long arrayLength) {
		return new LocalSize(arrayLength * 2);
	}
	
	
	/**
	 * Returns the size in bytes of an array of float values of the specified length.<br>
	 * @return <code>arrayLength * sizeof(float) = arrayLength * 4<br>
	 */
	public static LocalSize ofFloatArray(long arrayLength) {
		return new LocalSize(arrayLength * 4);
	}
	
	
	/**
	 * Returns the size in bytes of an array of double values of the specified length.<br>
	 * @return <code>arrayLength * sizeof(double) = arrayLength * 8<br>
	 */
	public static LocalSize ofDoubleArray(long arrayLength) {
		return new LocalSize(arrayLength * 8);
	}
	
		
	/**
	 * Returns the size in bytes of an array of T values of the specified length.<br>
	 */
	public static LocalSize ofArray(long arrayLength, Type componentType) {
		PointerIO io = PointerIO.getInstance(componentType);
		if (io == null)
			throw new RuntimeException("Unsupported type : " + Utils.toString(componentType));
		return new LocalSize(arrayLength * io.getTargetSize());
	}
}
